@@ -842,6 +842,7 @@ impl<'a> Tokenizer<'a> {
             } else if c == '$' {
                 if self.look_ahead_byte_matches('{') {
                     self.stream.skip_bytes(1);
+                    last_len = last_len.saturating_add(1);
                     self.curly_stack.push(OpenCurlyKind::Template);
                     if start == '`' {
                         return self.gen_template(
@@ -1708,6 +1709,19 @@ mod test {
         assert_eq!(item.start, 0);
         assert_eq!(item.end, 27);
     }
+
+    #[test]
+    fn tokenizer_regex_out_of_order() {
+        pretty_env_logger::try_init().ok();
+        let regex = r#"/((?:[^BEGHLMOSWYZabcdhmswyz']+)|(?:'(?:[^']|'')*')|(?:G{1,5}|y{1,4}|Y{1,4}|M{1,5}|L{1,5}|w{1,2}|W{1}|d{1,2}|E{1,6}|c{1,6}|a{1,5}|b{1,5}|B{1,5}|h{1,2}|H{1,2}|m{1,2}|s{1,2}|S{1,3}|z{1,4}|Z{1,5}|O{1,4}))([\s\S]*)/"#;
+        let mut t = Tokenizer::new(regex);
+        let next = t.next(true).unwrap();
+        let item = t.next_regex(next.end - next.start).unwrap();
+        assert_eq!(item.ty, RawToken::RegEx(211));
+        assert_eq!(item.start, 0);
+        assert_eq!(item.end, 211);
+    }
+
     #[test]
     #[should_panic = "new line in regex literal"]
     fn tokenizer_regex_new_line_negative() {
